
#define CreatePostScreenShader
// CreatePostScreenShader()
// This function creates a post screen shader and associates it with the 
// object that calls this function. You have to set up the variables 
// vertex_shader and pixel_shader before calling this function. The 
// variables camera_index and priority are optional. For more information 
// about the meaning of the single variables and post screen shaders in 
// general have a look at the help file.
u3d_index=external_call(global.u3d_create_post_screen_shader,vertex_shader,pixel_shader,camera_index,priority);
if(u3d_index!=0){
    u3d_type=14;
}
return false;


#define SetPSSTexture
// SetPSSTexture(TextureStage,TextureIndex)
// This function sets up a texture for the post screen shader object it's 
// called by. The texture is going to be used for the given texture stage. 
// Note that you can't set up another texture for stage 0, because this 
// stage always contains the scene texture. If you pass a negative value 
// for TextureIndex, the scene texture is going to be used for this stage, 
// too.
return external_call(global.u3d_set_pss_texture,u3d_index,argument0,argument1);


#define SetPSSVSConstant
// SetPSSVSConstant(VSConstantIndex,Value1,Value2,Value3,Value4)
// This function sets the given vertex shader constant register for the 
// post screen shader object that calls it to the given four values. 
// VSConstantIndex can be a value in the range from 0 to 96.
return external_call(global.u3d_set_pss_vs_constant,u3d_index,argument0,argument1,argument2,argument3,argument4);


#define SetPSSPSConstant
// SetPSSPSConstant(PSConstantIndex,Value1,Value2,Value3,Value4)
// This function sets the given pixel shader constant register for the 
// post screen shader object that calls it to the given four values.
// PSConstantIndex can be a value in the range from 0 to 8.
return external_call(global.u3d_set_pss_ps_constant,u3d_index,argument0,argument1,argument2,argument3,argument4);


#define SetPSSFilter
// SetPSSFilter(TextureStage,FilterType)
// This function changes the filter that is to be used for sampling  
// textures in the given stage when applying the post screen shader object 
// the function is called by. The given filter will be used for 
// minification and magnification of textures. To get information about 
// which value for FilterType refers to which filter have a look at 
// the help file.
return external_call(global.u3d_set_pss_filter,u3d_index,argument0,argument1);


#define SwitchShadows
// SwitchShadows(Enable)
// This function will enable or disable shadow casting for the light 
// source object it's being called by.
return external_call(global.u3d_switch_shadows,u3d_index,argument0);


#define SwitchShadowCasting
// SwitchShadowCasting(LightSourceID,Enable)
// This function enables or disables shadow casting for the light source 
// given by the first parameter. The change will apply to the model object 
// that calls this function. Note that only model objects can cast shadows.
return external_call(global.u3d_switch_shadow_casting,u3d_index,argument0.u3d_index,argument1);


#define CreateShadowOptimizedGeometry
// CreateShadowOptimizedGeometry(LODIndex,CreateConnectionPatches)
// This function creates seperate geometry for the given LOD of the model 
// that calls the function. The geometry will be optimal for shadow 
// casting. If you pass true for CreateConnectionPatches a lot more 
// geometry will need to be rendered, but the results will be better. This 
// is recommended especially for low poly models.
return external_call(global.u3d_create_shadow_optimized_geometry,u3d_index,argument0,argument1);


#define SwitchShadowReceiving
// SwitchShadowReceiving(LightSourceID,Enable)
// This function enables or disables receiving shadows from the given light 
// source for the object that calls the function. Note that primitives, 
// models and terrains can receive shadows.
return external_call(global.u3d_switch_shadow_receiving,u3d_type,u3d_index,argument0.u3d_index,argument1);


#define SetCustomTerrainLightMap
// SetCustomTerrainLightMap(TextureIndex)
// This function can be used to set up a custom light map for the terrain 
// that calls it. The custom light map will be used instead of the 
// automatically computed light map then. If TextureIndex doesn't refer to 
// a valid texture the automatically created light map will be used again.
return external_call(global.u3d_set_custom_terrain_light_map,u3d_index,argument0);


#define GetClippingPlaneSupport
// GetClippingPlaneSupport()
// This function returns the number of clipping planes the current graphics 
// device supports. Depending on the device the return value can vary 
// between 0 and 6. The return value minus one is the maximum clipping 
// plane index that can be passed to SetClippingPlane().
return external_call(global.u3d_get_clipping_plane_support);


#define GetShadowSupport
// GetShadowSupport()
// This function returns whether the graphics device that's currently being 
// used supports realtime stencil shadows.
return external_call(global.u3d_get_shadow_support);


#define GetDistanceToTerrain
// GetDistanceToTerrain(RayOriginX,RayOriginY,RayOriginZ,RayDirectionLongitude,RayDirectionLatitude,LODIndex)
// This function returns the distance from the point where the ray given by 
// the RayOrigin and RayDirection arguments meets the terrain the function 
// is called by to the point given by the RayOrigin arguments. If the ray 
// doesn't meet the terrain 100,000 is returned. Note that 
// GetTerrainHeightAtPos(...) should be used instead for a higher 
// efficience whenever possible.
return external_call(global.u3d_get_distance_to_terrain,argument0.u3d_index,argument1,argument2,argument3,argument4,argument5,argument6);


#define SetBoneFrame
// SetBoneFrame(BoneIndex,Frame,EffectChildBones)
// This function sets up a new frame to be used for the bone with the given 
// index. Frame has to be a positive value giving an absolute frame index. 
// If you pass true for EffectChildBones the frame value used for the bone 
// with the given index will be used for the bones that are appended to it 
// as well, else the child bones will use the default frame value of the 
// model or the frame value of another parent frame.
return external_call(global.u3d_set_bone_frame,u3d_index,argument0,argument1,argument2);


#define ExportToU3DFile
// ExportToU3DFile(OutputFile,Password,ExportResources)
// This function exports the object it's called by to the given output file 
// using the Ultimate 3D model file format. If you pass a valid string for 
// Password this password will be used to encrypt the file. If you pass 
// true for ExportResources all shader effects and texture files will be 
// saved to the *.u3d file as well.
return external_call(global.u3d_export_to_u3d_file,u3d_index,argument0,argument1,argument2);


#define SwitchAlphaBlending
// SwitchAlphaBlending(EnableAlphaBlending)
// This function enables or disables alpha blending for the primitive or 
// model object that calls it. If alpha blending is disabled anything 
// that's solid will be visible and anything that's partially transparent 
// will be invisible. This can lead to a significant performance 
// increasement because usually Ultimate 3D renders anything that's 
// transparent in a second pass. By default alpha blending is enabled.
return external_call(global.u3d_switch_alpha_blending,u3d_index,u3d_type,argument0);


#define CreatePortal
// CreatePortal(OriginX,OriginY,OriginZ,FacingDirectionLongitude,FacingDirectionLatitude,Radius,RoomIndex1,RoomIndex2)
// This function creates a new portal and returns it's index which is 
// needed to modify the portal after creating it. The first six parameters 
// define a circle with the point OriginX/Y/Z in it's center which faces 
// into the direction defined by FacingDirectionLongitude/Latitude and has 
// the given Radius. If a camera passes through the portal it's room index 
// will be changed from RoomIndex1 to RoomIndex2 if it comes from the 
// room RoomIndex1 or converse if it comes from the room RoomIndex2. For 
// more information about the portal engine refer to the help file.
return external_call(global.u3d_create_portal,argument0,argument1,argument2,argument3,argument4,argument5,argument6,argument7);


#define SetPortal
// SetPortal(PortalIndex,OriginX,OriginY,OriginZ,FacingDirectionLongitude,FacingDirectionLatitude,Radius,RoomIndex1,RoomIndex2)
// This function changes the portal with the given index. To get 
// information about the meaning of the different parameters refer to the 
// description of CreatePortal(...) or to the Ultimate 3D help file.
return external_call(global.u3d_set_portal,argument0,argument1,argument2,argument3,argument4,argument5,argument6,argument7,argument8);


#define OpenPortal
// OpenPortal(PortalIndex,OpenPortal)
// This function opens or closes the portal with the given index. If 
// OpenPortal is true it will get opened, if OpenPortal is false it will 
// get closed. If a portal is closed, the room behind it won't be visible 
// from another room. By default a portal is opened. For more information 
// about the portal engine refer to the help file.
return external_call(global.u3d_open_portal,argument0,argument1);


#define DestroyPortal
// DestroyPortal(PortalIndex)
// This function destroys the portal with the given index.
return external_call(global.u3d_destroy_portal,argument0);


#define SwitchPortalEngine
// SwitchPortalEngine(UsePortalEngine)
// This function turns the portal engine on or off for the camera object 
// that calls it. If UsePortalEngine is true the portal engine will be 
// used if it's false it won't be used. By default the portal engine isn't 
// used.
return external_call(global.u3d_switch_portal_engine,number,argument0);


#define GetRoomVisibility
// GetRoomVisibility(RoomIndex)
// This function returns true if the room with the given index can be 
// seen by the function that calls this function, or false if it can't be 
// seen.
return external_call(global.u3d_get_room_visibility,number,argument0);


#define GetObjectRoom
// GetObjectRoom()
// This function returns the room the object that calls this function 
// currently is in. For more information about rooms and the advanced 
// culling functions refer to the help file.
if(u3d_type==15){
    return external_call(global.u3d_get_object_room,u3d_type,number);
}
else{
    return external_call(global.u3d_get_object_room,u3d_type,u3d_index);
}


#define SwitchWireFrameMode
// SwitchWireFrameMode(EnableWireFrameMode)
// This function can be used to enable or disable wire frame mode. If 
// EnableWireFrameMode is true wire frame mode will be used for all 3D 
// objects, despite the background.
return external_call(global.u3d_switch_wire_frame_mode,argument0);


#define UpdateSkeleton
// UpdateSkeleton()
// Since the bone transformations only get update when Ultimate 3D renders 
// the matrices returned by GetBoneTransformation(...) usually are those 
// from the frame that can be currently seen on the screen. If you need 
// the transformations as they will be in the next frame you can use this 
// function to force Ultimate 3D to update the bone transformation matrices 
// for the object that calls this function. Note that this takes some 
// computing time.
return external_call(global.u3d_update_skeleton,u3d_index);


#define ReleaseModel
// ReleaseModel(ModelFile)
// This function match to PreloadMesh(...). Once Ultimate 3D has loaded a 
// model file, no matter whether this has been done through LoadMesh(...) 
// or PreloadMesh(...) Ultimate 3D keeps this model file in memory so it 
// doesn't need to load it again when it's needed again. An example: You 
// load a model using LoadMesh(...). Later this model object gets 
// destroyed. The model will be still in memory then. When another model 
// object gets created that uses this model file it won't have to load 
// anything. ReleaseModel(...) offers a possibility to force Ultimate 3D 
// to release a model. If there's still an object that's dependent on this 
// model file the function will return false and won't do anything. 
// Otherwise it releases the model and returns true. This reduces the 
// ammount of used pysical memory.
return external_call(global.u3d_release_model,argument0);


#define AddTextureCoordinateSetsFromModel
// AddModelTextureCoordinates(File,Password)
// This function adds all texture coordinate sets of the given model 
// (possibly an encrypted *.u3d file with the given password) to the object 
// that calls this function and to all objects that are based on the same 
// model (because geometry data doesn't get loaded multiple times, to 
// lower the memory usage). The model the texture coordinate set gets 
// added to must have the same number of meshes as the given model and 
// mesh i in the model the texture coordinates get added to must have the 
// same number of vertices as mesh i in the given model. This function is 
// very useful in context to light mapping. For more information have a 
// look at the help file.
return external_call(global.u3d_add_texture_coordinate_sets_from_model,u3d_index,argument0,argument1);


#define ComputeMatrixRotationAngles
// ComputeMatrixRotationAngles(OutputVectorID,InputMatrixID)
// This function computes rotx, roty and rotz values that describe the 
// rotation of the given matrix correctly. The computed values get saved 
// to the given output vector (rotx to x, roty to y, rotz to z, all in 
// degrees). If OutputVectorID is negative a new vector gets created and 
// it's ID gets returned. Never forget to call ReleaseVector(...) for 
// every vector you create.
return external_call(global.u3d_compute_matrix_rotation_angles,argument0,argument1);



#define GetRayTracingNormal
// GetRayTracingNormal(OutputVectorID)
// This function creates a vector containing the normal vector (a vector 
// that's perpendicular to a plane) of the triangle that has been hit by 
// the last ray tracing function (GetDistanceTo...(...)) and saves the 
// result using the given output vector. If a negative ID is passed for 
// OutputVectorID a new vector is created and it's ID gets returned. Never 
// forget to call ReleaseVector for every vector you create.
return external_call(global.u3d_get_ray_tracing_normal,argument0);


#define ScreenCoordToVector
// ScreenCoordToVector(OutputVectorID,ScreenCoordX,ScreenCoordY,CameraIndex)
// If you take a particular position on the screen everything that can 
// be seen at this position lies on one ray and this ray begins at the 
// camera position. This function computes a vector describing the 
// direction of the ray that belongs to the given screen space coordinate 
// (relative to the left top of the window) and saves the result using the 
// given output vector. If a negative ID gets passed for OutputVectorID a 
// new vector gets created and it's ID gets returned. Never forget to call 
// ReleaseVector for every vector you create.
return external_call(global.u3d_screen_coord_to_vector,argument0,argument1,argument2,argument3);

